--
-- Joskin water tank
-- Class for water tank
--
-- @author  PeterJ - euroDZN
-- @date  08/03/2012
--
-- @edited for FS13
-- @date  21/03/2013
--
-- http://eurodzn.wordpress.com/
--
-- Copyright (C) euroDZN, Confidential, All Rights Reserved.


water_tank = {};

function water_tank.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function water_tank:load(xmlFile)

	self.setIsTurnedOn = water_tank.setIsTurnedOn;
	self.getIsTurnedOnAllowed = water_tank.getIsTurnedOnAllowed;
	self.isTankUnloading = false;
	self.isTurnedOn = false;
	
	if self.isClient then
		local tankSound = getXMLString(xmlFile, "vehicle.tankSound#file");
		if tankSound ~= nil and tankSound ~= "" then
			tankSound = Utils.getFilename(tankSound, self.baseDirectory);
			self.tankSound = createSample("tankSound");
			self.tankSoundEnabled = false;
			loadSample(self.tankSound, tankSound, false);
			self.tankSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tankSound#pitchOffset"), 1);
			self.tankSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tankSound#volume"), 1);
		end;
	end;
	
	self.unloadParticleSystems = {};
	Utils.loadParticleSystem(xmlFile, self.unloadParticleSystems, "vehicle.ParticleSystems.unload", self.components, false, nil, self.baseDirectory);

	self.handleAnimation = getXMLString(xmlFile, "vehicle.unloading#animationName");
	self.handleNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.unloading#index"));
	self.planeAnimation = getXMLString(xmlFile, "vehicle.drinkingTrough#animationName");

	self.lastFillLevel = self.fillLevel;
	self.drinkSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.animals#drinkSpeed"), 0.0);
	if self.drinkSpeed > 0 then
		self.drinkSpeed = self.drinkSpeed * 2.3;
	end;
end;

function water_tank:delete()
	if self.tankSound ~= nil then
		delete(self.tankSound);
	end;
	Utils.deleteParticleSystem(self.unloadParticleSystems);
end;

function water_tank:readStream(streamId, connection)
	local turnedOn = streamReadBool(streamId);
	self:setIsTurnedOn(turnedOn, true);
end;
  
function water_tank:writeStream(streamId, connection)
	streamWriteBool(streamId, self.isTurnedOn);
end;

function water_tank:mouseEvent(posX, posY, isDown, isUp, button)
end;

function water_tank:keyEvent(unicode, sym, modifier, isDown)
end;

function water_tank:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local isTurnedOn = getXMLBool(xmlFile, key.."#drinkTrough");
		if isTurnedOn ~= nil then
			self:setIsTurnedOn(isTurnedOn);
		end;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function water_tank:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'drinkTrough="'..tostring(self.isTurnedOn)..'"';
	return attributes, nil;
end;

function water_tank:update(dt)
	if self.isTurnedOn then
		local cowsStored = g_currentMission.husbandries.cow.totalNumAnimals;
		local sheepStored = g_currentMission.husbandries.sheep.totalNumAnimals;
		if cowsStored > 0 or sheepStored > 0 then
			local thirstyAnimals = cowsStored + sheepStored;
			local drinkSpeed = thirstyAnimals;
			if thirstyAnimals > 65 then
				drinkSpeed = 65;
			end;
			self:setFillLevel(self.fillLevel - (drinkSpeed / self.drinkSpeed), self.currentFillType);
		end;
		if self.fillLevel <= 0 and self.capacity ~= 0 then
			self:setIsTurnedOn(false);
		end;
		if self.lastMovedDistance > 0.75 then
			self:setIsTurnedOn(false);
		end;
	end;
	if not self:getIsActive() then
		if self.playerInRange and self.fillLevel > 0 then	
			if self.isTurnedOn then
				g_currentMission:addHelpButtonText(g_i18n:getText("drinking_off"), InputBinding.ACTIVATE_OBJECT);
			else
				g_currentMission:addHelpButtonText(g_i18n:getText("drinking_on"), InputBinding.ACTIVATE_OBJECT);
			end;
			if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
				self:setIsTurnedOn(not self.isTurnedOn);
			end;
		end;
	else
		if self.isTurnedOn and not self.isWaterTrailerFilling then
			if self:getIsActiveForInput() then
				if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
					self:setIsTurnedOn(false);				
				end;
			end;
		end;
	end;
	self.lastFillLevel = self.fillLevel;
end;

function water_tank:updateTick(dt)
	if g_currentMission.player ~= nil then
		local nearestDistance = 1.7;
		local x1,y1,z1 = getWorldTranslation(self.handleNode);
		local x2,y2,z2 = getWorldTranslation(g_currentMission.player.rootNode);
		local distance = Utils.vector3Length(x1-x2,y1-y2,z1-z2);
		if distance < nearestDistance then
			self.playerInRange = true;
		else
			self.playerInRange = false;
		end;
	end;
end;

function water_tank:draw()
end;

function water_tank:onAttach(attacherVehicle)
end;

function water_tank:onDetach()
	if self.deactivateOnDetach then
		water_tank.onDeactivate(self);
	else
		water_tank.onDeactivateSounds(self);
	end;
end;
  
function water_tank:onLeave()
	if self.deactivateOnLeave then
		water_tank.onDeactivate(self);
	else
		water_tank.onDeactivateSounds(self);
	end;
end;
  
function water_tank:onDeactivate()
	water_tank.onDeactivateSounds(self);
end;
  
function water_tank:onDeactivateSounds()
	if self.tankSoundEnabled then
		stopSample(self.tankSound);
		self.tankSoundEnabled = false;
	end;
end;

function water_tank:getIsTurnedOnAllowed(isTurnedOn)
	if not isTurnedOn or self.fillLevel > 0 then
		return true;
	end;
end;

function water_tank:setIsTurnedOn(isTurnedOn, noEventSend)
	if isTurnedOn ~= self.isTurnedOn then
		if self:getIsTurnedOnAllowed(isTurnedOn) then
			SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)
			self.isTurnedOn = isTurnedOn;
			if self.isTurnedOn then
				if self.planeAnimation ~= nil and self.playAnimation ~= nil then
					self:playAnimation(self.planeAnimation, 1, nil, true);
				end;
			else
				if self.planeAnimation ~= nil and self.playAnimation ~= nil then
					self:playAnimation(self.planeAnimation, -1, nil, true);
				end;		
			end;
		end;
	end;
end;
